@@ -1,12 +1,16 @@
 from django.conf import settings
-from django.utils import translation, six
+from django.template.defaultfilters import safe
+from django.utils import translation
+
+import floppyforms as forms
+
+from urllib.parse import urlencode
 
 try:
     from django.contrib.gis import gdal, geos
 except ImportError:
     """GDAL / GEOS not installed"""
 
-import floppyforms as forms
 
 __all__ = ('GeometryWidget', 'GeometryCollectionWidget',
            'PointWidget', 'MultiPointWidget',
@@ -59,16 +63,13 @@ class BaseGeometryWidget(forms.Textarea):
     def get_context(self, name, value, attrs=None, extra_context={}):
         # If a string reaches here (via a validation error on another
         # field) then just reconstruct the Geometry.
-        if isinstance(value, six.text_type):
+        if isinstance(value, str):
             try:
                 value = geos.GEOSGeometry(value)
             except (geos.GEOSException, ValueError):
                 value = None
 
-        if (
-            value and value.geom_type.upper() != self.geom_type and
-            self.geom_type != 'GEOMETRY'
-        ):
+        if value and value.geom_type.upper() != self.geom_type and self.geom_type != 'GEOMETRY':
             value = None
 
         # Defaulting the WKT value to a blank string
@@ -80,13 +81,18 @@ class BaseGeometryWidget(forms.Textarea):
                     ogr = value.ogr
                     ogr.transform(srid)
                     wkt = ogr.wkt
-                except gdal.OGRException:
+                except gdal.GDALException:
                     pass  # wkt left as an empty string
             else:
                 wkt = value.wkt
         context = super(BaseGeometryWidget, self).get_context(name, wkt, attrs)
         context['module'] = 'map_%s' % name.replace('-', '_')
         context['name'] = name
+
+        # Deprecated, we will remove this in a future release, it's no longer
+        # used. But we keep it here for now as it's documented in
+        # docs/geodjango.rst
+
         # Django >= 1.4 doesn't have ADMIN_MEDIA_PREFIX anymore, we must
         # rely on contrib.staticfiles.
         if hasattr(settings, 'ADMIN_MEDIA_PREFIX'):
@@ -140,32 +146,40 @@ class BaseMetacartaWidget(BaseGeometryWidget):
 
     class Media:
         js = (
-            'http://openlayers.org/api/OpenLayers.js',
+            'floppyforms/openlayers/OpenLayers.js',
             'floppyforms/js/MapWidget.js',
         )
 
 
 class BaseOsmWidget(BaseGeometryWidget):
     """An OpenStreetMap base widget"""
-    map_srid = 900913
+    map_srid = 3857
     template_name = 'floppyforms/gis/osm.html'
 
     class Media:
         js = (
-            'http://openlayers.org/api/OpenLayers.js',
-            'http://www.openstreetmap.org/openlayers/OpenStreetMap.js',
+            'floppyforms/openlayers/OpenLayers.js',
+            'https://www.openstreetmap.org/openlayers/OpenStreetMap.js',
             'floppyforms/js/MapWidget.js',
         )
 
 
 class BaseGMapWidget(BaseGeometryWidget):
     """A Google Maps base widget"""
-    map_srid = 900913
+    map_srid = 3857
     template_name = 'floppyforms/gis/google.html'
+    google_maps_api_key = None
+
+    @property
+    def media(self):
+        qs_dict = {'v': '3'}
+        if self.google_maps_api_key is not None:
+            qs_dict['key'] = self.google_maps_api_key
 
-    class Media:
         js = (
-            'http://openlayers.org/api/OpenLayers.js',
+            'floppyforms/openlayers/OpenLayers.js',
             'floppyforms/js/MapWidget.js',
-            'http://maps.google.com/maps/api/js?sensor=false',
+            # Needs (safe) because may contain "&":
+            safe('https://maps.google.com/maps/api/js?' + urlencode(qs_dict))
         )
+        return forms.Media(js=js)
