@@ -1,14 +1,19 @@
 from collections import defaultdict
 from contextlib import contextmanager
 
+import django
 from django.conf import settings
-from django.forms.util import ErrorList
 from django.template import (Library, Node, Variable,
                              TemplateSyntaxError, VariableDoesNotExist)
 from django.template.base import token_kwargs
-from django.template.loader import get_template
 from django.utils.functional import empty
 
+from ..compat import get_template
+
+
+from django.forms.utils import ErrorList
+
+
 register = Library()
 
 
@@ -30,6 +35,49 @@ def is_bound_field(var):
     return all(hasattr(var, attr) for attr in significant_attributes)
 
 
+def raise_or_not_variable_does_not_exist_compat_version(context):
+    """This function raise exception if template is in debug mode. If not, it return ''.
+    Multiples ways (depending to the Django version) to retrieve the value of template debug.
+
+    :param context: context for the field.
+    :return: Either '' or raise Exception
+    """
+    if django.VERSION < (1, 8):
+        if settings.TEMPLATE_DEBUG:
+            raise
+        else:
+            return ''
+
+    if django.VERSION >= (1, 8) and django.VERSION < (1, 10):
+        from django.template.engine import Engine
+        template_debug_value = getattr(settings, 'TEMPLATE_DEBUG', None)
+        if template_debug_value:
+            raise
+        if not template_debug_value:
+            return ''
+        if template_debug_value is None:
+            try:
+                engine = context.template.engine
+            except AttributeError:
+                engine = Engine.get_default()
+            if engine.debug:
+                raise
+            return ''
+
+    if django.VERSION >= (1, 10):
+        from django.template.engine import Engine
+        try:
+            engine = context.template.engine
+        except AttributeError:
+            engine = Engine.get_default()
+        if engine.debug:
+            raise
+
+        return ''
+
+    raise
+
+
 class ConfigFilter(object):
     """
     Can be used as ``filter`` argument to ``FormConfig.configure()``. This
@@ -139,7 +187,8 @@ class FormConfig(object):
 
         """
         if filter is None:
-            filter = lambda **kwargs: True
+            def filter(**kwargs):
+                return True
         self.dicts[-1][key].append((value, filter))
 
     def retrieve(self, key, **kwargs):
@@ -211,7 +260,7 @@ class BaseFormNode(Node):
         while bits and bits[0] not in ('using', 'with', 'only'):
             variables.append(Variable(bits.pop(0)))
         if not variables:
-            raise TemplateSyntaxError(u'%s tag expectes at least one '
+            raise TemplateSyntaxError('%s tag expectes at least one '
                                       'template variable as argument.' %
                                       tagname)
         return variables
@@ -329,17 +378,15 @@ class ModifierBase(BaseFormNode):
             try:
                 for_ = self.options['for'].resolve(context)
             except VariableDoesNotExist:
-                if settings.TEMPLATE_DEBUG:
-                    raise
-                return ''
+                return raise_or_not_variable_does_not_exist_compat_version(context)
+
             filter = ConfigFilter(for_)
         if self.options['using']:
             try:
                 template_name = self.options['using'].resolve(context)
             except VariableDoesNotExist:
-                if settings.TEMPLATE_DEBUG:
-                    raise
-                return ''
+                return raise_or_not_variable_does_not_exist_compat_version(context)
+
             config.configure(self.template_config_name,
                              template_name, filter=filter)
         if self.options['with']:
@@ -348,7 +395,7 @@ class ModifierBase(BaseFormNode):
                 for name, var in self.options['with'].items()])
             config.configure(self.context_config_name,
                              extra_context, filter=filter)
-        return u''
+        return ''
 
     @classmethod
     def parse_bits(cls, tagname, modifier, bits, parser, tokens):
@@ -443,10 +490,19 @@ class BaseFormRenderNode(BaseFormNode):
                 template_name = self.options['using'].resolve(context)
             else:
                 template_name = self.get_template_name(context)
-            return get_template(template_name)
-        except:
-            if settings.TEMPLATE_DEBUG:
-                raise
+            return get_template(context, template_name)
+        except:  # noqa: E722
+            if django.VERSION < (1, 10):
+                if settings.DEBUG:
+                    raise
+            else:
+                try:
+                    engine = context.template.engine
+                except AttributeError:
+                    if settings.DEBUG:
+                        raise
+                if engine.debug:
+                    raise
 
     def get_extra_context(self, context):
         variables = []
@@ -508,7 +564,7 @@ class FormNode(BaseFormRenderNode):
         if not hasattr(var, '__iter__'):
             return False
         if is_formset(var):
-                return True
+            return True
         if is_form(var):
             return False
         # form duck-typing was not successful so it must be a list
@@ -615,9 +671,7 @@ class FormFieldNode(BaseFormRenderNode):
         try:
             bound_field = self.variables[0].resolve(context)
         except VariableDoesNotExist:
-            if settings.DEBUG:
-                raise
-            return u''
+            return raise_or_not_variable_does_not_exist_compat_version(context)
 
         widget = config.retrieve('widget', bound_field=bound_field)
         extra_context = self.get_extra_context(context)
@@ -627,10 +681,7 @@ class FormFieldNode(BaseFormRenderNode):
             try:
                 template_name = self.options['using'].resolve(context)
             except VariableDoesNotExist:
-                if settings.DEBUG:
-                    raise
-                return u''
-
+                return raise_or_not_variable_does_not_exist_compat_version(context)
         if self.options['only']:
             context_instance = context.new(extra_context)
         else:
@@ -688,7 +739,7 @@ class WidgetNode(Node):
             widget_ctx = {'field': field}
             template = 'floppyforms/dummy.html'
 
-        template = get_template(template)
+        template = get_template(context, template)
         context.update(widget_ctx)
         rendered = template.render(context)
         context.pop()
