@@ -1,27 +1,25 @@
-from itertools import chain
-import re
 import datetime
+import re
+from itertools import chain
 
 import django
 from django import forms
-try:
-    from django.forms.utils import to_current_timezone
-except ImportError:
-    # Fall back to old module name for Django <= 1.5
-    from django.forms.util import to_current_timezone
-from django.forms.widgets import FILE_INPUT_CONTRADICTION
 from django.conf import settings
+from django.forms.widgets import FILE_INPUT_CONTRADICTION
 from django.template import loader
-from django.utils.html import conditional_escape
-from django.utils.translation import ugettext_lazy as _
-from django.utils import datetime_safe, formats, six
+from django.utils import datetime_safe, formats
 from django.utils.dates import MONTHS
-from django.utils.encoding import force_text
+from django.utils.encoding import force_str
+from django.utils.html import conditional_escape
 from django.utils.safestring import mark_safe
+from django.utils.translation import gettext_lazy as _
 
 from .compat import MULTIVALUE_DICT_TYPES, flatten_contexts
 
 
+from django.forms.utils import to_current_timezone
+
+
 RE_DATE = re.compile(r'(\d{4})-(\d\d?)-(\d\d?)$')
 
 
@@ -40,11 +38,34 @@ __all__ = (
 class Widget(forms.Widget):
     is_required = False
 
+    def render(self, name, value, attrs=None, renderer=None):
+        """
+        Returns this Widget rendered as HTML, as a Unicode string.
+        The 'value' given is not guaranteed to be valid input, so subclass
+        implementations should program defensively.
+        """
+        raise NotImplementedError('subclasses of Widget must provide a render() method')
+
+    def build_attrs(self, extra_attrs=None, **kwargs):
+        """
+        Backported from Django 1.10
+        Helper function for building an attribute dictionary.
+        """
+        attrs = dict(self.attrs, **kwargs)
+        if extra_attrs:
+            attrs.update(extra_attrs)
+        return attrs
+
     # Backported from Django 1.7
     @property
     def is_hidden(self):
         return self.input_type == 'hidden' if hasattr(self, 'input_type') else False
 
+    # Backported from Django 1.9
+    if not hasattr(forms.Widget, 'format_value'):
+        def format_value(self, value):
+            return self._format_value(value)
+
 
 class Input(Widget):
     template_name = 'floppyforms/input.html'
@@ -66,19 +87,21 @@ class Input(Widget):
     def get_context_data(self):
         return {}
 
-    def _format_value(self, value):
+    def format_value(self, value):
         if self.is_localized:
             value = formats.localize_input(value)
-        return force_text(value)
+        return force_str(value)
 
     def get_context(self, name, value, attrs=None):
         context = {
+            'widget': self,
             'type': self.input_type,
             'name': name,
             'hidden': self.is_hidden,
             'required': self.is_required,
             'True': True,
         }
+
         # True is injected in the context to allow stricter comparisons
         # for widget attrs. See #25.
         if self.is_hidden:
@@ -89,7 +112,7 @@ class Input(Widget):
 
         if value != '':
             # Only add the value if it is non-empty
-            context['value'] = self._format_value(value)
+            context['value'] = self.format_value(value)
 
         context.update(self.get_context_data())
         context['attrs'] = self.build_attrs(attrs)
@@ -110,7 +133,7 @@ class Input(Widget):
         template_name = kwargs.pop('template_name', None)
         if template_name is None:
             template_name = self.template_name
-        context = self.get_context(name, value, attrs=attrs or {}, **kwargs)
+        context = self.get_context(name, value, attrs=attrs or {})
         context = flatten_contexts(self.context_instance, context)
         return loader.render_to_string(template_name, context)
 
@@ -133,10 +156,10 @@ class PasswordInput(TextInput):
         super(PasswordInput, self).__init__(attrs)
         self.render_value = render_value
 
-    def render(self, name, value, attrs=None):
+    def render(self, name, value, attrs=None, renderer=None):
         if not self.render_value:
             value = None
-        return super(PasswordInput, self).render(name, value, attrs)
+        return super(PasswordInput, self).render(name, value, attrs, renderer=renderer)
 
 
 class HiddenInput(Input):
@@ -150,7 +173,7 @@ class MultipleHiddenInput(HiddenInput):
         super(MultipleHiddenInput, self).__init__(attrs)
         self.choices = choices
 
-    def render(self, name, value, attrs=None, choices=()):
+    def render(self, name, value, attrs=None, choices=(), renderer=None):
         if value is None:
             value = []
 
@@ -163,7 +186,7 @@ class MultipleHiddenInput(HiddenInput):
                 input_attrs['id'] = '%s_%s' % (id_, i)
             input_ = HiddenInput()
             input_.is_required = self.is_required
-            inputs.append(input_.render(name, force_text(v), input_attrs))
+            inputs.append(input_.render(name, force_str(v), input_attrs, renderer=renderer))
         return mark_safe("\n".join(inputs))
 
     def value_from_datadict(self, data, files, name):
@@ -178,7 +201,7 @@ class SlugInput(TextInput):
     """<input type="text"> validating slugs with a pattern"""
     def get_context(self, name, value, attrs):
         context = super(SlugInput, self).get_context(name, value, attrs)
-        context['attrs']['pattern'] = "[-\w]+"
+        context['attrs']['pattern'] = r"[-\w]+"
         return context
 
 
@@ -186,8 +209,8 @@ class IPAddressInput(TextInput):
     template_name = 'floppyforms/ipaddress.html'
 
     """<input type="text"> validating IP addresses with a pattern"""
-    ip_pattern = ("(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25"
-                  "[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}")
+    ip_pattern = (r"(25[0-5]|2[0-4]\d|[0-1]?\d?\d)(\.(25"
+                  r"[0-5]|2[0-4]\d|[0-1]?\d?\d)){3}")
 
     def get_context(self, name, value, attrs):
         context = super(IPAddressInput, self).get_context(name, value, attrs)
@@ -201,11 +224,11 @@ class FileInput(Input):
     needs_multipart_form = True
     omit_value = True
 
-    def render(self, name, value, attrs=None):
+    def render(self, name, value, attrs=None, renderer=None):
         if self.omit_value:
             # File inputs can't render an existing value if it's not saved
             value = None
-        return super(FileInput, self).render(name, value, attrs=attrs)
+        return super(FileInput, self).render(name, value, attrs=attrs, renderer=renderer)
 
     def value_from_datadict(self, data, files, name):
         return files.get(name, None)
@@ -256,7 +279,7 @@ class ClearableFileInput(FileInput):
             return False
         return upload
 
-    def _format_value(self, value):
+    def format_value(self, value):
         # If the value is falsy, then it might be a file instance with no file
         # associated with. That can happen if you get the value from a
         # models.ImageField that is set to None. In that case we just return
@@ -278,8 +301,8 @@ class Textarea(Input):
             default_attrs.update(attrs)
         super(Textarea, self).__init__(default_attrs)
 
-    def _format_value(self, value):
-        return conditional_escape(force_text(value))
+    def format_value(self, value):
+        return conditional_escape(force_str(value))
 
 
 class DateInput(Input):
@@ -291,10 +314,10 @@ class DateInput(Input):
         super(DateInput, self).__init__(attrs)
         # We hardcode the format here as the HTML5 input type="date" only
         # accepts the ISO date format. See issue #115 for details:
-        # https://github.com/gregmuellegger/django-floppyforms/issues/115
+        # https://github.com/jazzband/django-floppyforms/issues/115
         self.format = '%Y-%m-%d'
 
-    def _format_value(self, value):
+    def format_value(self, value):
         if hasattr(value, 'strftime'):
             value = datetime_safe.new_date(value)
             return value.strftime(self.format)
@@ -326,7 +349,7 @@ class DateTimeInput(Input):
             self.format = formats.get_format('DATETIME_INPUT_FORMATS')[0]
             self.manual_format = False
 
-    def _format_value(self, value):
+    def format_value(self, value):
         if hasattr(value, 'strftime'):
             value = datetime_safe.new_datetime(value)
             return value.strftime(self.format)
@@ -358,7 +381,7 @@ class TimeInput(Input):
             self.format = formats.get_format('TIME_INPUT_FORMATS')[0]
             self.manual_format = False
 
-    def _format_value(self, value):
+    def format_value(self, value):
         if hasattr(value, 'strftime'):
             return value.strftime(self.format)
         return value
@@ -442,11 +465,11 @@ class CheckboxInput(Input, forms.CheckboxInput):
             context['attrs']['checked'] = True
         return context
 
-    def _format_value(self, value):
+    def format_value(self, value):
         if value in ('', True, False, None):
             value = None
         else:
-            value = force_text(value)
+            value = force_str(value)
         return value
 
     def value_from_datadict(self, data, files, name):
@@ -470,7 +493,7 @@ class Select(Input):
 
     def get_context(self, name, value, attrs=None, choices=()):
         if not hasattr(value, '__iter__') or isinstance(value,
-                                                        six.string_types):
+                                                        str):
             value = [value]
         context = super(Select, self).get_context(name, value, attrs)
 
@@ -493,10 +516,10 @@ class Select(Input):
             if isinstance(option_label, (list, tuple)):
                 group = []
                 for val, lab in option_label:
-                    group.append((force_text(val), lab))
+                    group.append((force_str(val), lab))
                 groups.append((option_value, group))
             else:
-                option_value = force_text(option_value)
+                option_value = force_str(option_value)
                 if groups and groups[-1][0] is None:
                     groups[-1][1].append((option_value, option_label))
                 else:
@@ -504,10 +527,10 @@ class Select(Input):
         context["optgroups"] = groups
         return context
 
-    def _format_value(self, value):
+    def format_value(self, value):
         if len(value) == 1 and value[0] is None:
             return []
-        return set(force_text(v) for v in value)
+        return set(force_str(v) for v in value)
 
 
 class NullBooleanSelect(Select):
@@ -517,7 +540,7 @@ class NullBooleanSelect(Select):
                    ('3', _('No')))
         super(NullBooleanSelect, self).__init__(attrs, choices)
 
-    def _format_value(self, value):
+    def format_value(self, value):
         value = value[0]
         try:
             value = {True: '2', False: '3', '2': '2', '3': '3'}[value]
@@ -546,10 +569,10 @@ class NullBooleanSelect(Select):
 class SelectMultiple(Select):
     allow_multiple_selected = True
 
-    def _format_value(self, value):
+    def format_value(self, value):
         if len(value) == 1 and value[0] is None:
             value = []
-        return [force_text(v) for v in value]
+        return set(force_str(v) for v in value)
 
     def value_from_datadict(self, data, files, name):
         if isinstance(data, MULTIVALUE_DICT_TYPES):
@@ -564,8 +587,8 @@ class SelectMultiple(Select):
                 data = []
             if len(initial) != len(data):
                 return True
-            initial_set = set([force_text(value) for value in initial])
-            data_set = set([force_text(value) for value in data])
+            initial_set = set([force_str(value) for value in initial])
+            data_set = set([force_str(value) for value in data])
             return data_set != initial_set
 
 
@@ -583,6 +606,101 @@ class MultiWidget(forms.MultiWidget):
     def is_hidden(self):
         return all(w.is_hidden for w in self.widgets)
 
+    def build_attrs(self, base_attrs, extra_attrs=None, **kwargs):
+        """
+        Backported from Django 1.10
+        Helper function for building an attribute dictionary.
+        """
+        attrs = dict(self.attrs, **kwargs)
+        attrs.update(base_attrs)
+        if extra_attrs:
+            attrs.update(extra_attrs)
+        return attrs
+
+    if django.VERSION < (1, 11):
+        # backport
+        def format_value(self, value):
+            """
+            Return a value as it should appear when rendered in a template.
+            """
+            if value == '' or value is None:
+                return None
+            if self.is_localized:
+                return formats.localize_input(value)
+            return force_str(value)
+
+        # backport port
+        def get_context(self, name, value, attrs):
+            # context = super(MultiWidget, self).get_context(name, value, attrs)
+            # the following is an inline version of the previous call
+            context = {}
+            context['widget'] = {
+                'name': name,
+                'is_hidden': self.is_hidden,
+                'required': self.is_required,
+                'value': self.format_value(value),
+                'attrs': self.build_attrs(self.attrs, attrs),
+                'template_name': self.template_name,
+            }
+            if self.is_localized:
+                for widget in self.widgets:
+                    widget.is_localized = self.is_localized
+            # value is a list of values, each corresponding to a widget
+            # in self.widgets.
+            if not isinstance(value, list):
+                value = self.decompress(value)
+
+            final_attrs = context['widget']['attrs']
+            input_type = final_attrs.pop('type', None)
+            id_ = final_attrs.get('id')
+            subwidgets = []
+            for i, widget in enumerate(self.widgets):
+                if input_type is not None:
+                    widget.input_type = input_type
+                widget_name = '%s_%s' % (name, i)
+                try:
+                    widget_value = value[i]
+                except IndexError:
+                    widget_value = None
+                if id_:
+                    widget_attrs = final_attrs.copy()
+                    widget_attrs['id'] = '%s_%s' % (id_, i)
+                else:
+                    widget_attrs = final_attrs
+                subwidgets.append(widget.get_context(widget_name, widget_value, widget_attrs)['widget'])
+            context['widget']['subwidgets'] = subwidgets
+            return context
+
+    def render(self, name, value, attrs=None, **kwargs):
+        context = self.get_context(name, value, attrs)
+        if self.is_localized:
+            for widget in self.widgets:
+                widget.is_localized = self.is_localized
+        # value is a list of values, each corresponding to a widget
+        # in self.widgets.
+        if not isinstance(value, list):
+            value = self.decompress(value)
+
+        final_attrs = context['widget']['attrs']
+        input_type = final_attrs.pop('type', None)
+        id_ = final_attrs.get('id')
+        rendered_template = ''
+        for i, widget in enumerate(self.widgets):
+            if input_type is not None:
+                widget.input_type = input_type
+            widget_name = '%s_%s' % (name, i)
+            try:
+                widget_value = value[i]
+            except IndexError:
+                widget_value = None
+            if id_:
+                widget_attrs = final_attrs.copy()
+                widget_attrs['id'] = '%s_%s' % (id_, i)
+            else:
+                widget_attrs = final_attrs
+            rendered_template += widget.render(widget_name, widget_value, widget_attrs)
+        return rendered_template
+
 
 class SplitDateTimeWidget(MultiWidget):
     supports_microseconds = False
@@ -661,12 +779,12 @@ class SelectDateWidget(forms.Widget):
         context['attrs'] = attrs
         return context
 
-    def render(self, name, value, attrs=None, extra_context={}):
+    def render(self, name, value, attrs=None, extra_context={}, renderer=None):
         try:
             year_val, month_val, day_val = value.year, value.month, value.day
         except AttributeError:
             year_val = month_val = day_val = None
-            if isinstance(value, six.string_types):
+            if isinstance(value, str):
                 if settings.USE_L10N:
                     try:
                         input_format = formats.get_format(
