@@ -11,7 +11,7 @@ from django.template.loader import render_to_string
 from django.test import TestCase
 from django.test.utils import override_settings
 from django.utils.dates import MONTHS
-from django.utils.encoding import python_2_unicode_compatible
+
 from django.utils.timezone import now
 
 import floppyforms as forms
@@ -25,7 +25,6 @@ skipIf = unittest.skipIf
 skipUnless = unittest.skipUnless
 
 
-@python_2_unicode_compatible
 class SomeModel(models.Model):
     some_field = models.CharField(max_length=255)
 
@@ -635,7 +634,7 @@ class WidgetRenderingTest(TestCase):
         self.assertHTMLEqual(rendered, """
         <p>
             <label for="id_multi">Multi:</label>
-            <select name="multi" id="id_multi" required multiple>
+            <select name="multi" id="id_multi" required multiple="multiple">
                 <option value="1">English</option>
                 <option value="12">Deutsch</option>
                 <option value="123" selected>Francais</option>
